-----------------------------------------------------------------------------------------------
-- Client Lua Script for TaskbarAlerts
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- TaskbarAlerts Module Definition
-----------------------------------------------------------------------------------------------
local TaskbarAlerts = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

local bOnAFK = true
local bOnPartyInvite = true
local bOnNeighborRequest = true
local bOnLFGFound = true
local bOnDuelRequest  = true
local bOnNewMail = true
local bOnWhisper = true
local bOnCombat = true
local bOnPvp = true
local bOnReadyCheck = true
local bOnGroupSummon = true

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function TaskbarAlerts:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function TaskbarAlerts:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- TaskbarAlerts OnLoad
-----------------------------------------------------------------------------------------------
function TaskbarAlerts:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("TaskbarAlerts.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- TaskbarAlerts OnDocLoaded
-----------------------------------------------------------------------------------------------
function TaskbarAlerts:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "TaskbarAlertsForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("ta", "OnTaskbarAlertsOn", self)
		Apollo.RegisterEventHandler("ChatMessage",                   "OnChatMessage",                   self)
		Apollo.RegisterEventHandler("Group_Invited",                 "OnGroupInvite",                   self)
		Apollo.RegisterEventHandler("Group_ReadyCheck",              "OnReadyCheck",                    self)
		Apollo.RegisterEventHandler("HousingNeighborInviteRecieved", "OnHousingNeighborInviteRecieved", self)
		Apollo.RegisterEventHandler("MatchingGameReady",             "OnMatchingGameReady",             self)
		Apollo.RegisterEventHandler("MatchingRoleCheckStarted",      "OnMatchingRoleCheckStarted",      self)
		Apollo.RegisterEventHandler("DuelStateChanged",              "OnDuelStateChanged",              self)
		Apollo.RegisterEventHandler("AvailableMail",                 "OnAvailableMail",                 self)
		Apollo.RegisterEventHandler("EnteredCombat",                 "OnEnteredCombat",                 self)
		
		-- Do additional Addon initialization here
	end
end

function TaskbarAlerts:OnSave(nLevel)
	if nLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	local tSave = {
		kOnAFK = bOnAFK,
		kOnPartyInvite = bOnPartyInvite,
		kOnNeighborRequest = bOnNeighborRequest,
		kOnLFGFound = bOnLFGFound,
		kOnDuelRequest = bOnDuelRequest,
		kOnNewMail = bOnNewMail,
		kOnWhisper = bOnWhisper,
		kOnCombat = bOnCombat,
		kOnPvp = bOnPvp,
		kOnReadyCheck = bOnReadyCheck,
		kOnGroupSummon = bOnGroupSummon,
		}
	return tSave
end

function TaskbarAlerts:OnRestore(nLevel, tSave)
	if nLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	if tSave.kOnAFK ~= nil then bOnAFK = tSave.kOnAFK end
	if tSave.kOnPartyInvite ~= nil then bOnPartyInvite = tSave.kOnPartyInvite end
	if tSave.kOnNeighborRequest ~= nil then bOnNeighborRequest = tSave.kOnNeighborRequest end
	if tSave.kOnLFGFound ~= nil then bOnLFGFound = tSave.kOnLFGFound end
	if tSave.kOnDuelRequest ~= nil then bOnDuelRequest = tSave.kOnDuelRequest end
	if tSave.kOnNewMail ~= nil then bOnNewMail = tSave.kOnNewMail end
	if tSave.kOnWhisper ~= nil then bOnWhisper = tSave.kOnWhisper end
	if tSave.kOnCombat ~= nil then bOnCombat = tSave.kOnCombat end
	if tSave.kOnPvp ~= nil then bOnPvp = tSave.kOnPvp end
	if tSave.kOnReadyCheck ~= nil then bOnReadyCheck = tSave.kOnReadyCheck end
	if tSave.kOnGroupSummon ~= nil then bOnGroupSummon = tSave.kOnGroupSummon end
end

-----------------------------------------------------------------------------------------------
-- TaskbarAlerts Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/ta"
function TaskbarAlerts:OnTaskbarAlertsOn()
	local afkButton = self.wndMain:FindChild("AFKToggleButton")
	local partyInvButton = self.wndMain:FindChild("PartyInviteToggleButton")
	local neighborReqButton = self.wndMain:FindChild("NeighborRequestToggleButton")
	local LFGButton = self.wndMain:FindChild("LFGFoundToggleButton")
	local duelReqButton = self.wndMain:FindChild("DuelRequestToggleButton")
	local newMailButton = self.wndMain:FindChild("NewMailToggleButton")
	local whisperButton = self.wndMain:FindChild("WhisperToggleButton")
	local combatButton = self.wndMain:FindChild("CombatToggleButton")
	local pvpButton = self.wndMain:FindChild("PvpToggleButton")
	local readyCheckButton = self.wndMain:FindChild("ReadyCheckButton")
	local groupSummonButton = self.wndMain:FindChild("GroupSummonButton")
	
	afkButton:SetCheck(bOnAFK)
	afkButton:SetData(1)
	partyInvButton:SetCheck(bOnPartyInvite)
	partyInvButton:SetData(2)
	neighborReqButton:SetCheck(bOnNeighborRequest)
	neighborReqButton:SetData(3)
	LFGButton:SetCheck(bOnLFGFound)
	LFGButton:SetData(4)
	duelReqButton:SetCheck(bOnDuelRequest)
	duelReqButton:SetData(5)
	newMailButton:SetCheck(bOnNewMail)
	newMailButton:SetData(6)
	whisperButton:SetCheck(bOnWhisper)
	whisperButton:SetData(7)
	combatButton:SetCheck(bOnCombat)
	combatButton:SetData(8)
	pvpButton:SetCheck(bOnPvp)
	pvpButton:SetData(9)
	readyCheckButton:SetCheck(bOnReadyCheck)
	readyCheckButton:SetData(10)
	groupSummonButton:SetCheck(bOnGroupSummon)
	groupSummonButton:SetData(11)
		
	self.wndMain:Show(true)
end


-----------------------------------------------------------------------------------------------
-- TaskbarAlertsForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function TaskbarAlerts:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function TaskbarAlerts:OnCancel()
	self.wndMain:Close() -- hide the window
end


function TaskbarAlerts:OnToggleChecked(wndHandler, wndControl, eMouseButton)
	local nButtonId = wndControl:GetData()
	if nButtonId and nButtonId == 1 then
		bOnAFK = true
	elseif nButtonId and nButtonId == 2 then
		bOnPartyInvite = true
	elseif nButtonId and nButtonId == 3 then
		bOnNeighborRequest = true
	elseif nButtonId and nButtonId == 4 then
		bOnLFGFound = true
	elseif nButtonId and nButtonId == 5 then
		bOnDuelRequest = true
	elseif nButtonId and nButtonId == 6 then
		bOnNewMail = true
	elseif nButtonId and nButtonId == 7 then
		bOnWhisper = true
	elseif nButtonId and nButtonId == 8 then
		bOnCombat = true
	elseif nButtonId and nButtonId == 9 then
		bOnPvp = true
	elseif nButtonId and nButtonId == 10 then
		bOnReadyCheck = true
	elseif nButtonId and nButtonId == 11 then
		bOnGroupSummon = true
	end
end

function TaskbarAlerts:OnToggleUnchecked(wndHandler, wndControl, eMouseButton)
	local nButtonId = wndControl:GetData()
	if nButtonId and nButtonId == 1 then
		bOnAFK = false
	elseif nButtonId and nButtonId == 2 then
		bOnPartyInvite = false
	elseif nButtonId and nButtonId == 3 then
		bOnNeighborRequest = false
	elseif nButtonId and nButtonId == 4 then
		bOnLFGFound = false
	elseif nButtonId and nButtonId == 5 then
		bOnDuelRequest = false
	elseif nButtonId and nButtonId == 6 then
		bOnNewMail = false
	elseif nButtonId and nButtonId == 7 then
		bOnWhisper = false
	elseif nButtonId and nButtonId == 8 then
		bOnCombat = false
	elseif nButtonId and nButtonId == 9 then
		bOnPvp = false
	elseif nButtonId and nButtonId == 10 then
		bOnReadyCheck = false
	elseif nButtonId and nButtonId == 11 then
		bOnGroupSummon = false
	end
end

function TaskbarAlerts:OnChatMessage(tChannel, tMessage, nNum)
	local strAfkMessage = Apollo.GetString(552503)
	if bOnAFK and tChannel:GetType() == ChatSystemLib.ChatChannel_System and tMessage.arMessageSegments[1].strText == strAfkMessage then
		Apollo.AlertAppWindow()
		return
	end
	if bOnWhisper and tChannel:GetType() == ChatSystemLib.ChatChannel_Whisper or tChannel:GetType() == ChatSystemLib.ChatChannel_AccountWhisper then
		Apollo.AlertAppWindow()
		return
	end
	if bOnPvp then
	 	if tChannel:GetType() == ChatSystemLib.ChatChannel_System then
			local kw = "$1c"
			local kwLen = kw:len()
			local kwPos = string.find(Apollo.GetString(564188), kw, 1, true)
			local checkPrefix = false
			local checkSuffix = false
			if kwPos and kwPos > 1 then checkPrefix = true end
			if kwPos and kwPos < string.len(Apollo.GetString(564188)) then checkSuffix = true end
			local prefix
			local suffix
			local prefixMatches = 0
			local suffixMatches = 0
			if checkPrefix then
				prefix = string.sub(Apollo.GetString(564188), 1, kwPos - 1, 1, true)
				prefixMatches = string.find(tMessage.arMessageSegments[1].strText, prefix)
				if not prefixMatches then prefixMatches = 0 end
			end
			if checkSuffix then
				suffix = string.sub(Apollo.GetString(564188), kwPos + kwLen, string.len(Apollo.GetString(564188)))
				suffixMatches = string.find(tMessage.arMessageSegments[1].strText, suffix, 1, true)
				if not suffixMatches then suffixMatches = 0 end
			end
			if checkPrefix and prefixMatches < 1 then return end
			if checkSuffix and suffixMatches < 1 then return end
			-- If we get to this point, then both prefix and suffix are a match, if compared, so we can just alert now
			Apollo.AlertAppWindow()
			return
		end
	end
	if bOnGroupSummon then
		if tChannel:GetType() == ChatSystemLib.ChatChannel_Datachron then
			-- need to find Id for summon string, and alter for keyword
			local kw = "$(summoner.Name)"
			local kwLen = kw:len()
			local kwPos = string.find(Apollo.GetString(566459), kw, 1, true)
			local checkPrefix = false
			local checkSuffix = false
			if kwPos and kwPos > 1 then checkPrefix = true end
			if kwPos and kwPos < string.len(Apollo.GetString(566459)) then checkSuffix = true end
			local prefix
			local suffix
			local prefixMatches = 0
			local suffixMatches = 0
			if checkPrefix then
				prefix = string.sub(Apollo.GetString(566459), 1, kwPos - 1)
				prefixMatches = string.find(tMessage.arMessageSegments[1].strText, prefix, 1, true)
				if not prefixMatches then prefixMatches = 0 end
			end
			if checkSuffix then
				suffix = string.sub(Apollo.GetString(566459), kwPos + kwLen, string.len(Apollo.GetString(566459)))
				suffixMatches = string.find(tMessage.arMessageSegments[1].strText, suffix, 1, true)
				if not suffixMatches then suffixMatches = 0 end
			end
			if checkPrefix and prefixMatches < 1 then return end
			if checkSuffix and suffixMatches < 1 then return end
			-- If we get to this point, then both prefix and suffix are a match, if compared, so we can just alert now
			Apollo.AlertAppWindow()
			return
		end
	end
end

function TaskbarAlerts:OnGroupInvite()
	if bOnPartyInvite then Apollo.AlertAppWindow() end
end

function TaskbarAlerts:OnReadyCheck()
	if bOnReadyCheck then Apollo.AlertAppWindow() end
end

function TaskbarAlerts:OnHousingNeighborInviteRecieved()
	if bOnNeighborRequest then Apollo.AlertAppWindow() end
end

function TaskbarAlerts:OnMatchingGameReady()
	if bOnLFGFound then Apollo.AlertAppWindow() end
end

function TaskbarAlerts:OnMatchingRoleCheckStarted()
	if bOnLFGFound then Apollo.AlertAppWindow() end
end

function TaskbarAlerts:OnDuelStateChanged()
	if bOnDuelRequest then Apollo.AlertAppWindow() end
end

function TaskbarAlerts:OnAvailableMail()
	if bOnNewMail then Apollo.AlertAppWindow() end
end

function TaskbarAlerts:OnEnteredCombat()
	if bOnCombat then Apollo.AlertAppWindow() end
end
-----------------------------------------------------------------------------------------------
-- TaskbarAlerts Instance
-----------------------------------------------------------------------------------------------
local TaskbarAlertsInst = TaskbarAlerts:new()
TaskbarAlertsInst:Init()
