-----------------------------------------------------------------------------------------------
-- Client Lua Script for TrackTor
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- TrackTor Module Definition
-----------------------------------------------------------------------------------------------
local TrackTor = {}
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local version = 2.5
local revision = ""
 
-----------------------------------------------------------------------------------------------
-- TrackTor variables
-----------------------------------------------------------------------------------------------
local activeItem = 0
local tItems = {}
local tOptions = {}
local tMarkers = {}

local Overlay
local ReportWindow
local AlertWindow

local player
local playerVec
local playerPos

----- Tables
tUnits = {}
tUnits.players = {}
tUnits.playerIDsWithBuffs = {}
tUnits.nonPlayers = {}
tUnits.psiPoints = {}

-----------------------------------------------------------------------------------------------
-- SavedData
-----------------------------------------------------------------------------------------------
local tSavedData = {}

function TrackTor:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function TrackTor:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
	
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	
	----- Event Handlers
	Apollo.RegisterEventHandler("UnitCreated",  					"OnUnitCreated", self)
    Apollo.RegisterEventHandler("UnitDestroyed",					"OnUnitDestroyed", self)
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", 		"OnInterfaceMenuListHasLoaded", self)
	Apollo.RegisterEventHandler("ToggleTrackTor", 					"OnToggleTrackTor", self)
	Apollo.RegisterEventHandler("TargetUnitChanged",                "OnTargetUnitChanged", self)
end

-----------------------------------------------------------------------------------------------
-- TrackTor OnLoad
-----------------------------------------------------------------------------------------------
function TrackTor:OnLoad()
    self.xmlDoc = XmlDoc.CreateFromFile("TrackTor.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- TrackTor OnDocLoaded
-----------------------------------------------------------------------------------------------
function TrackTor:OnDocLoaded()
	if self.xmlDoc == nil or not self.xmlDoc:IsLoaded() then return end

	GeminiColor = Apollo.GetPackage("GeminiColor")
	if GeminiColor ~= nil then
		GeminiColor_Package = GeminiColor.tPackage
		if GeminiColor_Package ~= nil then
			self.picker = GeminiColor_Package:CreateColorPicker(self, "ColorPickerCallback", true, "ffff0000", "some", 4, "userdefined", "values")
			self.picker:Show(false)
		end
	end

	self.wndMain = Apollo.LoadForm(self.xmlDoc, "TrackTorForm", nil, self)
	if self.wndMain == nil then return end 
	
	self.wndMain:Show(false, true)
	
	Overlay = Apollo.LoadForm(self.xmlDoc, "Overlay", "InWorldHudStratum", self)
	Overlay:Show(true, true)
	
	ReportWindow = Apollo.LoadForm(self.xmlDoc, "ReportWindow", "InWorldHudStratum", self)
	
	--AlertWindow = Apollo.LoadForm(self.xmlDoc, "AlertWindow", "InWorldHudStratum", self)

	self:Initialize()
end

-----------------------------------------------------------------------------------------------
-- TrackTor Initialization
-----------------------------------------------------------------------------------------------
function TrackTor:Initialize()
	--Adding saved data
	if tSavedData ~= nil then
		self:Restore()
	end
	
	--- Something amazing!!!
	if tSavedData.about ~= nil then
		self.wndMain:FindChild("About"):SetCheck(tSavedData.about)
	end
	
	self.timer = ApolloTimer.Create(0.5, start, "OnTimer", self)
	self.timerTenth = ApolloTimer.Create(0.1, start, "UpdateAlertWindows", self)
	
	--- Other EventHandlers
	Apollo.RegisterEventHandler("NextFrame", 						"OnNextFrame", self)
	
	Apollo.RegisterSlashCommand("tracktor", 						"OnToggleTrackTor", self)
end

-----------------------------------------------------------------------------------------------
-- TrackTor Logic
-----------------------------------------------------------------------------------------------
function TrackTor:Logic()
	Overlay:DestroyAllPixies()

	for itemId, item in pairs(tItems) do
		if item:FindChild("Button"):IsChecked() then
		
			----- Target
			if tOptions[itemId]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "Target" then
			local target = GameLib.GetTargetUnit()
				if target then
					if tOptions[itemId]:FindChild("DeadCheckBox"):IsChecked() == false and target:GetHealth() == 0 then
					
					else
						if tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Line" then
							self:DrawLine(itemId, target)
						elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Circle" then
							self:DrawCircle(itemId, target)
						elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Marker" then
							self:DrawMarker(itemId, target)
						end
					end
				end
			end
			
			----- Focus
			if tOptions[itemId]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "Focus" then
			if player then
				local target = player:GetAlternateTarget()
					if target  then
						if tOptions[itemId]:FindChild("DeadCheckBox"):IsChecked() == false and target:GetHealth() == 0 then
						
						else
							if tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Line" then
								self:DrawLine(itemId, target)
							elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Circle" then
								self:DrawCircle(itemId, target)
							elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Marker" then
								self:DrawMarker(itemId, target)
							end
						end
					end
				end
			end
			
			----- Player
			if tOptions[itemId]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "Player" then
				units = {}
				sUnits = tOptions[item:GetId()]:FindChild("PlayerNameEditBox"):GetText()
				if sUnits ~= "" then
					endIndex = string.find(sUnits, ";")
					tIndex = 0
					
					if endIndex == nil then
						units[0] = sUnits
					else
						while endIndex ~= nil do
							units[tIndex] = string.sub(sUnits, 0, endIndex - 1)
							tIndex = tIndex + 1
							
							sUnits = string.sub(sUnits, endIndex + 1, string.len(sUnits))
							endIndex = string.find(sUnits, ";")
						end
						units[tIndex] = string.sub(sUnits, 0, string.len(sUnits))
					end
					for i, u in pairs(units) do
						local unit = GameLib.GetPlayerUnitByName(u)
						if unit then
							if tOptions[itemId]:FindChild("DeadCheckBox"):IsChecked() == false and unit:GetHealth() == 0 then
								
							else
								if tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Line" then
									self:DrawLine(itemId, unit)
								elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Circle" then
									self:DrawCircle(itemId, unit)
								elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Marker" then
									self:DrawMarker(itemId, unit)
								end
							end
						end
					end
				end
			end
			
			----- Casts
			if tOptions[itemId]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "Casts" then
				units = {}
				sUnits = tOptions[item:GetId()]:FindChild("CastsEditBox"):GetText()
				if sUnits ~= "" then
					endIndex = string.find(sUnits, ";")
					tIndex = 0
					
					if endIndex == nil then
						units[0] = sUnits
					else
						while endIndex ~= nil do
							units[tIndex] = string.sub(sUnits, 0, endIndex - 1)
							tIndex = tIndex + 1
							
							sUnits = string.sub(sUnits, endIndex + 1, string.len(sUnits))
							endIndex = string.find(sUnits, ";")
						end
						units[tIndex] = string.sub(sUnits, 0, string.len(sUnits))
					end
					for i, u in pairs(units) do
						for i, unit in pairs(tUnits.players) do
							if unit:ShouldShowCastBar() then
								--Print(unit:GetCastName() .. " " .. u)
								if unit:GetCastName() == u then
									if tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Line" then
										self:DrawLine(itemId, unit)
									elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Circle" then
										self:DrawCircle(itemId, unit)
									elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Marker" then
										self:DrawMarker(itemId, unit)
									end
									
									if tOptions[itemId]:FindChild("ReportCastsCheckBox"):IsChecked() then
										self:AddToReportWindow(tOptions[itemId]:FindChild("NameEditBox"):GetText(), unit:GetName())
									end
									
									if tOptions[itemId]:FindChild("AlertCastsCheckBox"):IsChecked() then
										self:ShowAlertWindow(tOptions[itemId]:FindChild("NameEditBox"):GetText(), 30)
									end
								end
							end
						end
					end
				end
			end
			
			----- Weapon
			if tOptions[itemId]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "Weapon" then
			local target = tUnits.weapon
				if target then
					if tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Line" then
						self:DrawLine(itemId, target)
					elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Circle" then
						self:DrawCircle(itemId, target)
					elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Marker" then
						self:DrawMarker(itemId, target)
					end
				end
			end

			----- Buffs/Debuffs
			if tOptions[itemId]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "Buffs/Debuffs" then
				if tUnits.playerIDsWithBuffs[itemId] then
					for i, id in pairs(tUnits.playerIDsWithBuffs[itemId]) do
						if tUnits.players[id] then
							if tOptions[itemId]:FindChild("DeadCheckBox"):IsChecked() == false and tUnits.players[id]:GetHealth() == 0 then
								
							else
								if tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Line" then
									self:DrawLine(itemId, tUnits.players[id])
								elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Circle" then
									self:DrawCircle(itemId, tUnits.players[id])
								elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Marker" then
									self:DrawMarker(itemId, tUnits.players[id])
								end
								
								if tOptions[itemId]:FindChild("ReportBuffsCheckBox"):IsChecked() then
									self:AddToReportWindow(tOptions[itemId]:FindChild("NameEditBox"):GetText(), tUnits.players[id]:GetName())
								end
							end
						end
					end
				end
			end	
			
			----- NonPlayers
			if tOptions[itemId]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "Entities" then
				units = {}
				sUnits = tOptions[itemId]:FindChild("EntitiesEditBox"):GetText()
				if sUnits ~= "" then
					endIndex = string.find(sUnits, ";")
					tIndex = 0
					
					if endIndex == nil then
						units[0] = sUnits
					else
						while endIndex ~= nil do
							units[tIndex] = string.sub(sUnits, 0, endIndex - 1)
							tIndex = tIndex + 1
							
							sUnits = string.sub(sUnits, endIndex + 1, string.len(sUnits))
							endIndex = string.find(sUnits, ";")
						end
						units[tIndex] = string.sub(sUnits, 0, string.len(sUnits))
					end
					for i, u in pairs(units) do
						local tE = tUnits.nonPlayers[u]
						if tE then
							for i, e in pairs(tE) do
								if tOptions[itemId]:FindChild("DeadCheckBox"):IsChecked() == false and e:GetHealth() == 0 then
									return
								end
							
								if tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Line" then
									self:DrawLine(itemId, e)
								elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Circle" then
									self:DrawCircle(itemId, e)
								elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Marker" then
									self:DrawMarker(itemId, e)
								end
							end
						end
					end
				end
			end	
			
			----- PSI Points
			if tOptions[itemId]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "PSI Points" then
				for i, e in pairs(tUnits.psiPoints) do
					if tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Line" then
						self:DrawLine(itemId, e)
					elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Circle" then
						self:DrawCircle(itemId, e)
					elseif tOptions[itemId]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Marker" then
						self:DrawMarker(itemId, e)
					end
				end
			end	
			----- end end
		end
	end
	
	self:DrawReportWindow()
end
	
----------------------------------------------------------------------------------------------
-- Drawing
-----------------------------------------------------------------------------------------------

----- Lines
function TrackTor:DrawLine(itemId, unit)
	if unit== nil then return end

	local tPos = unit:GetPosition()
	local tVec = Vector3.New(tPos.x, tPos.y, tPos.z)
	local targetPos = GameLib.WorldLocToScreenPoint(tVec)
	
	if tOptions[itemId]:FindChild("DottedLineCheckBox"):IsChecked() then
		for i = 1, 19 do
			local fraction = i / 20
			local pos = GameLib.WorldLocToScreenPoint(Vector3.InterpolateLinear(playerVec, tVec, fraction))
			
			--- Outline
			if tOptions[itemId]:FindChild("LineOutlineCheckBox"):IsChecked() then
				tMarkers[itemId] = Overlay:AddPixie({bLine = false, strSprite = "WhiteCircle",
					cr = ApolloColor.new("ff000000"),
					loc = {nOffsets = {	pos.x - tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) / 2 - 1,
									pos.y - tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) / 2 - 1,
									pos.x + tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) + 1,
									pos.y + tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) + 1}}})
			end
					
			--- Dots
			tMarkers[itemId] = Overlay:AddPixie({bLine = false, strSprite = "WhiteCircle",
				cr = ApolloColor.new(tOptions[itemId]:FindChild("ColorEditBox"):GetText()),
				loc = {nOffsets = {	pos.x - tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) / 2,
									pos.y - tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) / 2,
									pos.x + tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()),
									pos.y + tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText())}}})
		end
	else
		--- Outline
		if tOptions[itemId]:FindChild("LineOutlineCheckBox"):IsChecked() then
			tMarkers[itemId] = Overlay:AddPixie({bLine = true, fWidth = tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) + 2,
				cr = ApolloColor.new("ff000000"),
				loc = {nOffsets = {playerPos.x, playerPos.y, targetPos.x, targetPos.y}}})
	
		end
	
		--- Line
		tMarkers[itemId] = Overlay:AddPixie({bLine = true, fWidth = tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()),
			cr = ApolloColor.new(tOptions[itemId]:FindChild("ColorEditBox"):GetText()),
			loc = {nOffsets = {playerPos.x, playerPos.y, targetPos.x, targetPos.y}}})
	end
end

----- Circles
function TrackTor:DrawCircle(itemId, unit)
	if unit == nil then return end

	local originPosition = unit:GetPosition()
	local originPositionVector = Vector3.New(originPosition.x, originPosition.y, originPosition.z)
	
	local dist = tOptions[itemId]:FindChild("CircleSizeText"):GetText()
	local resolution = tOptions[itemId]:FindChild("CircleResolutionText"):GetText()
	
	if GameLib.WorldLocToScreenPoint(originPositionVector).x < dist * -100 or GameLib.WorldLocToScreenPoint(originPositionVector).x > Overlay:GetWidth() + dist * 100 then return end
	if GameLib.WorldLocToScreenPoint(originPositionVector).y < dist * -100 or GameLib.WorldLocToScreenPoint(originPositionVector).y > Overlay:GetHeight() + dist * 100 then return end
	
	local sin = math.sin
    local cos =  math.cos
    local rad = math.rad

	local uface = unit:GetFacing()
    local uangle = math.atan2(uface.x, uface.z)
    
	local firstVector = Vector3.New(originPositionVector.x + dist * sin(uangle), originPositionVector.y, originPositionVector.z + dist * cos(uangle))
    local previousVector = firstVector
    local startPoint = GameLib.WorldLocToScreenPoint(previousVector)
	
	if tOptions[itemId]:FindChild("DottedLineCheckBox"):IsChecked() then
		for i = 1, resolution do
			local rotation_point = uangle + rad(-360 / (resolution / i))
			
			--- Outline
			if tOptions[itemId]:FindChild("LineOutlineCheckBox"):IsChecked() then
				tMarkers[itemId] = Overlay:AddPixie({bLine = false, strSprite = "WhiteCircle",
					cr = ApolloColor.new("ff000000"),
					loc = {nOffsets = {	startPoint.x - tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) / 2 - 1,
										startPoint.y - tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) / 2 - 1,
										startPoint.x + tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) + 1,
										startPoint.y + tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) + 1}}})
			end

			
			--- Dots
			tMarkers[itemId] = Overlay:AddPixie({bLine = false, strSprite = "WhiteCircle",
				cr = ApolloColor.new(tOptions[itemId]:FindChild("ColorEditBox"):GetText()),
				loc = {nOffsets = {	startPoint.x - tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) / 2,
									startPoint.y - tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) / 2,
									startPoint.x + tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()),
									startPoint.y + tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText())}}})
																
			local currentVector = Vector3.New(originPositionVector.x + dist * sin(rotation_point), originPositionVector.y, originPositionVector.z + dist * cos(rotation_point))
	        startPoint = GameLib.WorldLocToScreenPoint(currentVector)
		end
	else
		for i = 1, resolution do
	        local rotation_point = uangle + rad(-360 / (resolution / i))
	
	        local currentVector = Vector3.New(originPositionVector.x + dist * sin(rotation_point), originPositionVector.y, originPositionVector.z + dist * cos(rotation_point))
	        local endPoint = GameLib.WorldLocToScreenPoint(currentVector)
	
			--- Outline
			if tOptions[itemId]:FindChild("LineOutlineCheckBox"):IsChecked() then
				tMarkers[itemId] = Overlay:AddPixie({bLine = true, fWidth = tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()) + 2,
					cr = ApolloColor.new("ff000000"),
					loc = {nOffsets = {startPoint.x, startPoint.y, endPoint.x, endPoint.y}}})
			end
			
			--- Line
	        tMarkers[itemId] = Overlay:AddPixie({bLine = true, fWidth = tonumber(tOptions[itemId]:FindChild("LineWidthText"):GetText()),
				cr = ApolloColor.new(tOptions[itemId]:FindChild("ColorEditBox"):GetText()),
				loc = {nOffsets = {startPoint.x, startPoint.y, endPoint.x, endPoint.y}}})
	
			previousVector = currentVector
	        startPoint = endPoint
	    end
	end
end

----- Markers
function TrackTor:DrawMarker(itemId, unit)
	if unit == nil then return end

	local tPos = unit:GetPosition()
	local tVec = Vector3.New(tPos.x, tPos.y, tPos.z)
	local targetPos = GameLib.WorldLocToScreenPoint(tVec)
	
	if targetPos.x < 0 or targetPos.x > Overlay:GetWidth() then return end
	if targetPos.y < 0 or targetPos.y > Overlay:GetHeight() then return end
	
	local nSize = 30
	
	-- ClientSprites:QuestJewel_SilverRing
	-- Crafting_CoordSprites:sprCoord_AdditivePreviewTiny
	-- CRB_MinimapSprites:sprMM_IndicatorRing
	
	tMarkers[itemId] = Overlay:AddPixie({
		cr = ApolloColor.new(tOptions[itemId]:FindChild("ColorEditBox"):GetText()),
		strSprite = "ClientSprites:QuestJewel_SilverRing",
		loc = {nOffsets = {targetPos.x - nSize, targetPos.y - nSize, targetPos.x + nSize, targetPos.y + nSize}}
	})
end


-----------------------------------------------------------------------------------------------
-- TrackTor Functions
-----------------------------------------------------------------------------------------------
----- Timer

function TrackTor:OnNextFrame()
	player = GameLib.GetPlayerUnit()
    if player ~= nil then
        local pPos = player:GetPosition()
        playerVec = Vector3.New(pPos.x, pPos.y, pPos.z)
        playerPos = GameLib.WorldLocToScreenPoint(playerVec)
    end
	
	self:Logic()
end

function TrackTor:OnCancel()
	self.wndMain:Close() -- hide the window
end

---------------------------------------------------------------------------------------------------
-- Items Functions
---------------------------------------------------------------------------------------------------

----- Add Item

function TrackTor:AddItem( wndHandler, wndControl, eMouseButton )
	local tmpItem = Apollo.LoadForm(self.xmlDoc, "Item", self.wndMain:FindChild("ItemContainer"), self)
	
	local tmpOptions = Apollo.LoadForm(self.xmlDoc, "Options", self.wndMain:FindChild("OptionsContainer"), self)
	tmpOptions:Show(false)
	
	tItems[tmpItem:GetId()] = tmpItem
	tOptions[tmpItem:GetId()] = tmpOptions
	
	self:SortItemList()
end

function TrackTor:AddExistedItem(item, option)
	-- Item
	local tmpItem = Apollo.LoadForm(self.xmlDoc, "Item", self.wndMain:FindChild("ItemContainer"), self)
	
	if item.checked then
		tmpItem:FindChild("Button"):SetCheck(item.checked)
	end if item.name then
		tmpItem:FindChild("ListItemText"):SetText(item.name)
	end
	
	-- Options
	local tmpOptions = Apollo.LoadForm(self.xmlDoc, "Options", self.wndMain:FindChild("OptionsContainer"), self)
	tmpOptions:Show(false)
	
	if option.name then
		tmpOptions:FindChild("NameEditBox"):SetText(option.name)
	end if option.color then
		tmpOptions:FindChild("ColorEditBox"):SetText(option.color)
	end if option.type then
		tmpOptions:FindChild("TypeDropdown"):FindChild("Dropdown"):SetText(option.type)
	end if option.style then
		tmpOptions:FindChild("StyleDropdown"):FindChild("Dropdown"):SetText(option.style)
	end if option.color then
		tmpOptions:FindChild("ColorSelectButton"):SetBGColor(option.color)
	end if option.dottedLine then
		tmpOptions:FindChild("DottedLineCheckBox"):SetCheck(option.dottedLine)
	end if option.trackDead then
		tmpOptions:FindChild("DeadCheckBox"):SetCheck(option.trackDead)
	end if option.reportBuffs then
		tmpOptions:FindChild("ReportBuffsCheckBox"):SetCheck(option.reportBuffs)
	end if option.alertCasts then
		tmpOptions:FindChild("AlertCastsCheckBox"):SetCheck(option.alertCasts)
	end if option.reportCasts then
		tmpOptions:FindChild("ReportCastsCheckBox"):SetCheck(option.reportCasts)
	end
	
	if option.playerName then
		tmpOptions:FindChild("PlayerNameEditBox"):SetText(option.playerName)
	end if option.buffs then
		tmpOptions:FindChild("BuffsEditBox"):SetText(option.buffs)
	end if option.entities then
		tmpOptions:FindChild("EntitiesEditBox"):SetText(option.entities)
	end if option.casts then
		tmpOptions:FindChild("CastsEditBox"):SetText(option.casts)
	end
	
	
	
	if option.outline then
		tmpOptions:FindChild("LineOutlineCheckBox"):SetCheck(option.outline)
	end if option.lineWidth then
		tmpOptions:FindChild("LineWidthText"):SetText(option.lineWidth)
		tmpOptions:FindChild("LineWidthSliderBar"):SetValue(tonumber(option.lineWidth))
	end if option.circleSize then
		tmpOptions:FindChild("CircleSizeText"):SetText(option.circleSize)
		tmpOptions:FindChild("CircleSizeSliderBar"):SetValue(tonumber(option.circleSize * 10))
	end if option.circleResolution then
		tmpOptions:FindChild("CircleResolutionText"):SetText(option.circleResolution)
		tmpOptions:FindChild("CircleResolutionSliderBar"):SetValue(tonumber(option.circleResolution))
	end
	
	--------------------------------
	tItems[tmpItem:GetId()] = tmpItem
	tOptions[tmpItem:GetId()] = tmpOptions
	
	self:SortItemList()
	self:OptionsRefresh(tmpItem:GetId())
end

----- Remove Item

function TrackTor:RemoveItem( wndHandler, wndControl, eMouseButton )
	if activeItem ~= 0 then
		tItems[activeItem]:Destroy()
		tOptions[activeItem]:Destroy()
	
		tItems[activeItem] = nil
		tOptions[activeItem] = nil

		activeItem = 0
	
		self:SortItemList()
	end
end

----- Sort Itemlist

function TrackTor:SortItemList()
	local index = 0;
	for i, item in pairs(tItems) do
		item:Move(3, 3+index*35, 190, 38)
		index = index + 1
	end
end

----- Others
function TrackTor:ListItemClicked( wndHandler, wndControl, eMouseButton )
	if wndControl:GetName() ~= "Button" then
		self:SetActiveItem(wndControl:GetParent():GetId())
		self:OptionsRefresh(wndControl:GetParent():GetId())
	end
end

function TrackTor:SetActiveItem(itemId)
	if activeItem ~= 0 then
		tOptions[activeItem]:Show(false)
	end
	activeItem = itemId
	tOptions[itemId]:Show(true)
end

local tReportWindow = {}
function TrackTor:AddToReportWindow(key, value)
	if tReportWindow[key] == nil then
		tReportWindow[key] = value
	else
		tReportWindow[key] = tReportWindow[key] .. ", " .. value
	end
end


function TrackTor:DrawReportWindow()
	local sReport = ""
	for key, value in pairs(tReportWindow) do
		sReport = sReport .. key .. ": " .. value .. "\n"
	end
	ReportWindow:FindChild("EditBox"):SetText(sReport)
	tReportWindow = {}
end

local AlertWindows = {}
function TrackTor:UpdateAlertWindows()
	for i, alert in pairs(AlertWindows) do
		alert:SetData(alert:GetData() - 1)
		if alert:GetData() <= 0 then
			alert:Destroy()
			AlertWindows[i] = nil
		end
	end
end

function TrackTor:ShowAlertWindow(text, time)
	if AlertWindows[text] == nil then
		local alert = Apollo.LoadForm(self.xmlDoc, "AlertWindow", nil, self)
		alert:SetData(time)
		alert:FindChild("Text"):SetText(text)
		alert:Show(true)
		
		AlertWindows[text] = alert
	end
end

---------------------------------------------------------------------------------------------------
-- Options Functions
---------------------------------------------------------------------------------------------------

function TrackTor:NameEditBoxChanged( wndHandler, wndControl, strText )
	tItems[activeItem]:FindChild("ListItemText"):SetText(wndControl:GetText())
end

function TrackTor:ColorEditBoxChanged( wndHandler, wndControl, strText )
	wndControl:GetParent():FindChild("ColorSelectButton"):SetBGColor(wndControl:GetText())
end

local divider2Offset
local divider2NTop
local divider2NBottom
local divider2NLeft
local divider2NRight
function TrackTor:OptionsRefresh(activeItem)
	if divider2Offset == nil then
		divider2Offset = 30
		divider2NLeft, divider2NTop, divider2NRight, divider2NBottom = tOptions[activeItem]:FindChild("Divider2"):GetAnchorOffsets()
	end

	----- Hide everything
	tOptions[activeItem]:FindChild("PlayerName"):Show(false)
	tOptions[activeItem]:FindChild("Buffs"):Show(false)
	tOptions[activeItem]:FindChild("Entities"):Show(false)
	tOptions[activeItem]:FindChild("Casts"):Show(false)
	
	----- Type dropdown	
	if tOptions[activeItem]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "Player" then
		tOptions[activeItem]:FindChild("PlayerName"):Show(true)
	end
	
	if tOptions[activeItem]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "Buffs/Debuffs" then
		tOptions[activeItem]:FindChild("Buffs"):Show(true)
	end
	
	if tOptions[activeItem]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "Entities" then
		tOptions[activeItem]:FindChild("Entities"):Show(true)
	end
	
	if tOptions[activeItem]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "Casts" then
		tOptions[activeItem]:FindChild("Casts"):Show(true)
	end
	
	----- Style dropdown
	tOptions[activeItem]:FindChild("Divider1"):Show(true, true)
	tOptions[activeItem]:FindChild("Divider2"):Show(true, true)
	if tOptions[activeItem]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Line" then
		tOptions[activeItem]:FindChild("Divider2"):SetAnchorOffsets(0, divider2NTop - divider2Offset, 0, divider2NBottom - divider2Offset)
		tOptions[activeItem]:FindChild("CircleOptions"):Show(false)
	end 
	
	if tOptions[activeItem]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Circle" then
		tOptions[activeItem]:FindChild("Divider2"):SetAnchorOffsets(0, divider2NTop, 0, divider2NBottom)
		tOptions[activeItem]:FindChild("CircleOptions"):Show(true)
	end 
	
	if tOptions[activeItem]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "None" then
		tOptions[activeItem]:FindChild("Divider1"):Show(false, true)
		tOptions[activeItem]:FindChild("Divider2"):SetAnchorOffsets(tOptions[activeItem]:FindChild("Divider1"):GetAnchorOffsets())
	end
	
	if tOptions[activeItem]:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText() == "Marker" then
		tOptions[activeItem]:FindChild("Divider1"):Show(false, true)
		tOptions[activeItem]:FindChild("Divider2"):SetAnchorOffsets(tOptions[activeItem]:FindChild("Divider1"):GetAnchorOffsets())
	end
end

function TrackTor:ColorSelect( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation )
	if self.picker ~= nil then
		self.picker:Show(true)
	end
end

function TrackTor:LineWidthSlideBarChanged( wndHandler, wndControl, fNewValue, fOldValue )
	tOptions[activeItem]:FindChild("LineWidthText"):SetText(math.floor(fNewValue+0.5))
end

function TrackTor:CircleResolutionSlideBarChanged( wndHandler, wndControl, fNewValue, fOldValue )
	tOptions[activeItem]:FindChild("CircleResolutionText"):SetText(math.floor(fNewValue+0.5))
end

function TrackTor:CircleSizeSlideBarChanged( wndHandler, wndControl, fNewValue, fOldValue )
	tOptions[activeItem]:FindChild("CircleSizeText"):SetText(math.floor(fNewValue+0.5)/10)
end

-----------------------------------------------------------------------------------------------
-- TrackTor OnSave / OnRestore
-----------------------------------------------------------------------------------------------
function TrackTor:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
	
	tSettings = {}
	
	tSettings.about = self.wndMain:FindChild("About"):IsChecked()
		
	tSettings["items"] = {}
	for i, item in pairs(tItems) do
		tSettings["items"][i] = {}
		tSettings["items"][i].checked = item:FindChild("Button"):IsChecked()
		tSettings["items"][i].name = item:FindChild("ListItemText"):GetText()
	end
	
	tSettings["options"] = {}
	for i, option in pairs(tOptions) do
		tSettings["options"][i] = {}
		tSettings["options"][i].name = option:FindChild("NameEditBox"):GetText()
		tSettings["options"][i].color = option:FindChild("ColorEditBox"):GetText()
		tSettings["options"][i].type = option:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText()
		tSettings["options"][i].style = option:FindChild("StyleDropdown"):FindChild("Dropdown"):GetText()
		tSettings["options"][i].dottedLine = option:FindChild("DottedLineCheckBox"):IsChecked()
		tSettings["options"][i].trackDead = option:FindChild("DeadCheckBox"):IsChecked()
		tSettings["options"][i].reportBuffs = option:FindChild("ReportBuffsCheckBox"):IsChecked()
		tSettings["options"][i].alertCasts = option:FindChild("AlertCastsCheckBox"):IsChecked()
		tSettings["options"][i].reportCasts = option:FindChild("ReportCastsCheckBox"):IsChecked()
		
		tSettings["options"][i].playerName = option:FindChild("PlayerNameEditBox"):GetText()
		tSettings["options"][i].buffs = option:FindChild("BuffsEditBox"):GetText()
		tSettings["options"][i].entities = option:FindChild("EntitiesEditBox"):GetText()
		tSettings["options"][i].casts = option:FindChild("CastsEditBox"):GetText()
		
		tSettings["options"][i].outline = option:FindChild("LineOutlineCheckBox"):IsChecked()
		tSettings["options"][i].lineWidth = option:FindChild("LineWidthText"):GetText()
		tSettings["options"][i].circleSize = option:FindChild("CircleSizeText"):GetText()
		tSettings["options"][i].circleResolution = option:FindChild("CircleResolutionText"):GetText()
	end

	return tSettings
end

function TrackTor:OnRestore(eType, tData)
	tSavedData = tData
end

function TrackTor:Restore()
	self.wndMain:FindChild("About"):SetTooltip("version: " .. version .. revision .. "  -  created by WintyBadass")

	if tSavedData["items"] then
		for i, item in pairs(tSavedData["items"]) do
			self:AddExistedItem(item, tSavedData["options"][i])
		end
	end
end

-----------------------------------------------------------------------------------------------
-- EventHandlers functions
-----------------------------------------------------------------------------------------------
function TrackTor:OnUnitCreated(unit)
	if unit and unit:IsValid() then
		--- weapon
		if player and unit:GetName() == player:GetName() .. "' Weapon" or player and unit:GetName() == player:GetName() .. "'s Weapon" then
			tUnits.weapon = unit
		end
		
		--- players
		if unit:GetType() == "Player" then
			tUnits.players[unit:GetId()] = unit
		end
		
		--- nonPlayers
		if unit:GetType() == "NonPlayer" or unit:GetType() == "Simple" then
			if tUnits.nonPlayers[unit:GetName()] == nil then
				tUnits.nonPlayers[unit:GetName()] = {}
			end
			tUnits.nonPlayers[unit:GetName()][unit:GetId()] = unit
		end
		-- psiPoints
		if unit:GetType() == "Trap" and unit:GetName() == "Psi Point" then
			tUnits.psiPoints[unit:GetId()] = unit
		end
	end
end
	
function TrackTor:OnUnitDestroyed(unit)
	if unit and unit:IsValid() then
		--- weapon
		if player and unit:GetName() == player:GetName() .. "' Weapon" or player and unit:GetName() == player:GetName() .. "'s Weapon" then
			tUnits.weapon = nil
		end
		
		--- players
		if unit:GetType() == "Player" then
			tUnits.players[unit:GetId()] = nil
		end
		
		--- nonPlayers
		if unit:GetType() == "NonPlayer" or unit:GetType() == "Simple" then
			if tUnits.nonPlayers[unit:GetName()] == nil then return end
			tUnits.nonPlayers[unit:GetName()][unit:GetId()] = nil
		end
		-- psiPoints
		if unit:GetType() == "Trap" and unit:GetName() == "Psi Point" then
			tUnits.psiPoints[unit:GetId()] = nil
		end
	end
end

function TrackTor:OnTargetUnitChanged(unit)
	
end

----- Interface menu list
function TrackTor:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "TrackTor", {"ToggleTrackTor", "", ""})
 
	self:UpdateInterfaceMenuAlerts()
end

function TrackTor:UpdateInterfaceMenuAlerts()
	--Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", "TrackTor", {self.fights > 0, nil, self.fights})
end

function TrackTor:OnToggleTrackTor()
	self.wndMain:Show(not self.wndMain:IsShown())
end
-----------------------------------------------------------------------------------------------
-- Dropdowns Functions
-----------------------------------------------------------------------------------------------
function TrackTor:DropdownClicked( wndHandler, wndControl, eMouseButton )
	wndControl:FindChild("List"):Show(true)
end

function TrackTor:DropdownListItemClicked( wndHandler, wndControl, eMouseButton )
	wndControl:GetParent():GetParent():Show(false)
	wndControl:GetParent():GetParent():GetParent():SetText(wndControl:GetText())
	
	self:OptionsRefresh(activeItem)
end

-----------------------------------------------------------------------------------------------
-- Whatever
-----------------------------------------------------------------------------------------------
function TrackTor:OnTimer() 
	if tItems == nil then return end
	
	tUnits.playerIDsWithBuffs = {}

	----- Players with buffs/debuffs
	for itemId, item in pairs(tItems) do
		tUnits.playerIDsWithBuffs[itemId] = {}
		if item:FindChild("Button"):IsChecked() and tOptions[itemId]:FindChild("TypeDropdown"):FindChild("Dropdown"):GetText() == "Buffs/Debuffs" then
			for i, unit in pairs(tUnits.players) do
				local tables = { 
					{ name = "debuffs", ar = "arHarmful" },
					{ name = "buffs", ar = "arBeneficial" } }
					
				for index, tbl in pairs(tables) do
					for index, effect in pairs(unit:GetBuffs()[tbl.ar]) do
						local name = effect.splEffect:GetName()
						units = {}
						sUnits = tOptions[item:GetId()]:FindChild("BuffsEditBox"):GetText()
						 if sUnits ~= "" then
							endIndex = string.find(sUnits, ";")
							tIndex = 0
					
							if endIndex == nil then
								units[0] = sUnits
							else
								while endIndex ~= nil do
									units[tIndex] = string.sub(sUnits, 0, endIndex - 1)
									tIndex = tIndex + 1
									
									sUnits = string.sub(sUnits, endIndex + 1, string.len(sUnits))
									endIndex = string.find(sUnits, ";")
								end
								units[tIndex] = string.sub(sUnits, 0, string.len(sUnits))
							end
							for i, u in pairs(units) do
								if name == u then
									tUnits.playerIDsWithBuffs[itemId][unit:GetId()] = unit:GetId()
								end
							end
						end
					end
				end
			end
		end
	end
end
	
function TrackTor:ColorPickerCallback(strColor)
    tOptions[activeItem]:FindChild("ColorEditBox"):SetText(strColor)
    tOptions[activeItem]:FindChild("ColorSelectButton"):SetBGColor(strColor)
end

---------------------------------------------------------------------------------------------------
-- TrackTorForm Functions
---------------------------------------------------------------------------------------------------

function TrackTor:OnReportWindowButton( wndHandler, wndControl, eMouseButton )
	ReportWindow:Show(not ReportWindow:IsShown(), true)
end

-----------------------------------------------------------------------------------------------
-- TrackTor Instance
-----------------------------------------------------------------------------------------------
local TrackTorInst = TrackTor:new()
TrackTorInst:Init()
